@@ -44,7 +44,7 @@ impl<'info> Validate<'info> for CastVote<'info> {
         assert_keys_eq!(self.vote.proposal, self.proposal);
         assert_keys_eq!(self.vote.voter, self.escrow.owner);
         invariant!(
-            self.proposal.get_state(&self.governor)? == ProposalState::Active,
+            self.proposal.get_state()? == ProposalState::Active,
             "proposal must be active"
         );
         Ok(())
