@@ -45,7 +45,15 @@ pub mod locked_voter {
 
     /// Stakes `amount` tokens into the [Escrow].
     #[access_control(ctx.accounts.validate())]
-    pub fn lock(ctx: Context<Lock>, amount: u64, duration: i64) -> ProgramResult {
+    pub fn lock<'info>(
+        ctx: Context<'_, '_, '_, 'info, Lock<'info>>,
+        amount: u64,
+        duration: i64,
+    ) -> ProgramResult {
+        if ctx.accounts.locker.params.whitelist_enabled {
+            ctx.accounts.check_whitelisted(ctx.remaining_accounts)?;
+        }
+
         ctx.accounts.lock(amount, duration)
     }
 
@@ -72,4 +80,27 @@ pub mod locked_voter {
     pub fn set_locker_params(ctx: Context<SetLockerParams>, params: LockerParams) -> ProgramResult {
         ctx.accounts.set_locker_params(params)
     }
+
+    /// Creates a new [LockerWhitelistEntry] to whitelist program from CPI.
+    #[access_control(ctx.accounts.validate())]
+    pub fn approve_program_lock_privilege(
+        ctx: Context<ApproveProgramLockPrivilege>,
+        bump: u8,
+    ) -> ProgramResult {
+        ctx.accounts.approve_program_lock_privilege(bump)
+    }
+
+    /// Close a [LockerWhitelistEntry] revoking program's CPI privilege.
+    #[access_control(ctx.accounts.validate())]
+    pub fn revoke_program_lock_privilege(
+        ctx: Context<RevokeProgramLockPrivilege>,
+    ) -> ProgramResult {
+        ctx.accounts.revoke_program_lock_privilege()
+    }
+}
+
+#[error]
+pub enum ErrorCode {
+    #[msg("CPI caller not whitelisted to invoke lock instruction.")]
+    ProgramNotWhitelisted,
 }
